@@ -2,5 +2,7 @@
 # -*- coding: utf-8 -*-
 """A pure Python3 implementation of the LSV2 protocol"""
 from .client import LSV2
+from .const import *
+from .translate_messages import *
 
-__version__ = '0.6.4'
+__version__ = '0.6.6'
